export CONDA_BUILD="1"
export PYTHONNOUSERSITE="1"
export CONDA_DEFAULT_ENV="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p"
export ARCH="ppc64le"
export PREFIX="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p"
export BUILD_PREFIX="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_build_env"
export SYS_PREFIX="/root/worker/containers/00003osegof/tmp/build/80754af9/mc"
export SYS_PYTHON="/root/worker/containers/00003osegof/tmp/build/80754af9/mc/bin/python"
export SUBDIR="linux-ppc64le"
export build_platform="linux-ppc64le"
export SRC_DIR="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/work"
export ROOT="/root/worker/containers/00003osegof/tmp/build/80754af9/mc"
export CONDA_PY="38"
export PY3K="1"
export PY_VER="3.8"
export STDLIB_DIR="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib/python3.8"
export SP_DIR="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib/python3.8/site-packages"
export PYTHON="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/bin/python"
export NPY_VER="1.16"
export CONDA_NPY="116"
export NPY_DISTUTILS_APPEND_FLAGS="1"
export PERL_VER="5.26"
export CONDA_PERL="5.26"
export LUA_VER="5"
export CONDA_LUA="5"
export R_VER="3.5"
export CONDA_R="3.5"
export PKG_NAME="pymongo"
export PKG_VERSION="3.11.1"
export PKG_BUILDNUM="0"
export PKG_BUILD_STRING="placeholder"
export PKG_HASH="1234567"
export RECIPE_DIR="/root/worker/containers/00003osegof/tmp/build/80754af9/combined_recipe"
export CPU_COUNT="160"
export LANG="en_US.UTF-8"
export LC_ALL="en_US.UTF-8"
export SHLIB_EXT=".so"
export PATH="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_build_env:/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_build_env/bin:/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p:/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/bin:/root/worker/containers/00003osegof/tmp/build/80754af9/mc/bin:/root/worker/containers/00003osegof/tmp/build/80754af9/mc/condabin:/opt/conda/bin:/opt/conda/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
export HOME="/root"
export PKG_CONFIG_PATH="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib/pkgconfig"
export CMAKE_GENERATOR="Unix Makefiles"
export LD_RUN_PATH="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib"
export BUILD="powerpc64le-conda_cos7-linux-gnu"
export ignore_build_only_deps="{'numpy', 'python'}"
export extend_keys="{'ignore_version', 'ignore_build_only_deps', 'extend_keys', 'pin_run_as_build'}"
export pin_run_as_build="OrderedDict([('python', {'max_pin': 'x.x', 'min_pin': 'x.x'}), ('r-base', {'max_pin': 'x.x', 'min_pin': 'x.x'}), ('libboost', {'max_pin': 'x.x.x'})])"
export channel_targets="defaults"
export cxx_compiler_version="7.3.0"
export macos_min_version="10.9"
export readline="8.0"
export gstreamer="1.14"
export fontconfig="2.13"
export c_compiler="gcc"
export gnu="2.12.2"
export nodejs="8.12"
export libnetcdf="4.6"
export r_base="3.5"
export noarch_python="python"
export serf="1.3.9"
export r_implementation="r-base"
export blas_impl="openblas"
export boost="1.73"
export g2clib="1.6"
export openssl="1.1.1"
export libwebp="1.0.0"
export bzip2="1.0"
export mkl="2019"
export expat="2.2"
export geos="3.8.0"
export gmp="6.1"
export llvm_variant="llvm"
export mpfr="4"
export fortran_compiler="gfortran"
export fortran_compiler_version="7.3.0"
export libffi="3.3"
export libpng="1.6"
export hdfeos2="2.20"
export libgd="2.2.5"
export proj4="5.2.0"
export macos_machine="x86_64-apple-darwin13.4.0"
export sqlite="3"
export harfbuzz="2.4"
export libgsasl="1.8"
export libtiff="4.1"
export cran_mirror="https://mran.microsoft.com/snapshot/2018-01-01"
export MACOSX_DEPLOYMENT_TARGET="10.9"
export proj="6.2.1"
export python_implementation="cpython"
export zlib="1.2"
export cairo="1.14"
export rust_compiler="rust"
export libdap4="3.19"
export icu="58"
export qtwebkit="true"
export libgdal="3.0"
export apr="1.6.3"
export libxml2="2.9"
export xz="5"
export cpu_optimization_target="nocona"
export lzo="2"
export VERBOSE_CM="VERBOSE=1"
export python_impl="cpython"
export openjpeg="2.3"
export gst_plugins_base="1.14"
export pixman="0.34"
export clang_variant="clang"
export rust_compiler_version="1.46.0"
export dbus="1"
export freetype="2.10"
export cxx_compiler="gxx"
export target_platform="linux-ppc64le"
export libxslt="1.1"
export VERBOSE_AT="V=1"
export boost_cpp="1.73"
export glib="2"
export libprotobuf="3.11.2"
export r_version="3.5.0"
export c_compiler_version="7.3.0"
export jpeg="9"
export hdf4="4.2"
export giflib="5"
export cyrus_sasl="2.1.26"
export hdfeos5="5.1"
export libkml="1.3"
export hdf5="1.10.6"
export openblas="0.3.3"
export CONDA_BUILD_STATE="BUILD"
export PIP_NO_BUILD_ISOLATION="False"
export PIP_NO_DEPENDENCIES="True"
export PIP_IGNORE_INSTALLED="True"
export PIP_CACHE_DIR="/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/pip_cache"
export PIP_NO_INDEX="True"
eval "$('/root/worker/containers/00003osegof/tmp/build/80754af9/mc/bin/python' -m conda shell.bash hook)"
conda activate "/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p"
conda activate --stack "/root/worker/containers/00003osegof/tmp/build/80754af9/pymongo_1606344861968/_build_env"
